/*
 * Boo Development Tools for the Eclipse IDE
 * Copyright (C) 2005 Rodrigo B. de Oliveira (rbo@acm.org)
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package aQute.bnd.plugin.editors;

import java.util.*;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.source.ISharedTextColors;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;

import aQute.bnd.make.*;
import aQute.lib.osgi.*;

public class BndScanner extends RuleBasedScanner {
    static RGB C_MACRO       = new RGB(0, 255, 0);
    static RGB C_ERROR       = new RGB(255, 0, 0);
    static RGB C_COMMENT     = new RGB(128, 0, 0);
    static RGB C_INSTRUCTION = new RGB(0, 0, 255);
    static RGB C_OPTION      = new RGB(0, 0, 255);
    static RGB C_DEFAULT     = new RGB(0, 0, 0);
    static RGB C_DIRECTIVE   = new RGB(60, 60, 255);
    static RGB C_PROPERTY    = new RGB(60, 60, 255);
    static RGB C_COMPONENT   = new RGB(60, 60, 255);
	
	static Token createBoldToken(ISharedTextColors manager, RGB rgb) {
		return new Token(new TextAttribute(manager.getColor(rgb), null,
				SWT.BOLD));
	}

	static Token createToken(ISharedTextColors manager, RGB rgb) {
		return new Token(new TextAttribute(manager.getColor(rgb)));
	}
	static class BndWordRule implements IRule {

		private Map<String,IToken>				_words	= new HashMap<String,IToken>();
		private StringBuffer	_buffer	= new StringBuffer();
		private IToken			other;

		public BndWordRule(ISharedTextColors manager) {
			other = createToken(manager, C_DEFAULT);
			addWords(Analyzer.headers, createBoldToken(manager, C_INSTRUCTION));
			addWords(Analyzer.options, createBoldToken(manager, C_OPTION));
			addWords(Analyzer.directives, createBoldToken(manager, C_DIRECTIVE));
			addWords(ServiceComponent.componentDirectives, createBoldToken(manager, C_COMPONENT));
		}


		private boolean isWordStart(char c) {
			return Character.isJavaIdentifierStart(c);
		}

		private boolean isWordPart(char c) {
			return Character.isJavaIdentifierPart(c) || c=='-';
		}

		public IToken evaluate(ICharacterScanner scanner) {
			int c = scanner.read();
			if (isWordStart((char) c) || c=='-') {
				_buffer.setLength(0);
				do {
					_buffer.append((char) c);
					c = scanner.read();
				} while (c != ICharacterScanner.EOF && isWordPart((char) c));
				scanner.unread();

				IToken token = (IToken) _words.get(_buffer.toString());
				if (token != null)
					return token;
				return other;
			}

			scanner.unread();
			return Token.UNDEFINED;

		}

		private void addWords(String[] words, IToken token) {
			for (int i = 0; i < words.length; ++i) {
				_words.put(words[i], token);
			}
		}

	}

	public BndScanner(ISharedTextColors manager) {
		IToken defaultToken = new Token(new TextAttribute(manager
				.getColor(C_DEFAULT)));

		IRule[] rules = new IRule[] {
				new WhitespaceRule(new IWhitespaceDetector() {
					public boolean isWhitespace(char c) {
						return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
					}
				}), 
                new BndWordRule(manager),
                new MacroRule(createToken(manager,C_MACRO)),
                new EndOfLineRule("#", createToken(manager, C_COMMENT)),
                new EndOfLineRule("\\ ",createBoldToken(manager, C_ERROR)),
                new EndOfLineRule("\\\t", createBoldToken(manager, C_ERROR)),
				};

		setRules(rules);
		setDefaultReturnToken(defaultToken);
	}
}
